var visitorLatitude;
var visitorLongitude;
var errorCode = null;
var map;
var visitorLatlng;
var newMarker;
var visitorInfowindow;
var destinationLocation;
var currentLocation;
var zoomingIn = false;
var latitude;
var longitude;
var altitude; 
var accuracy;
var altitudeAccuracy;
var speed;
var timestamp;
var positionData;
var newLocationIcon;
var markerBounds;
var enabled = true;

$(function () {

	// Get the current position from the browser
	if (!Modernizr.geolocation) {	

		visitorLatitude = geoip_latitude();
		visitorLongitude = geoip_longitude();
		
		positionData = null;	
		
		// Init the Google map
		initialize();	
	
	} else {
	
		// Use html5 API
		navigator.geolocation.getCurrentPosition(onPositionReady, onError);
	}
	
	function onPositionReady(position) {

		if (errorCode == null) {	

			// Get geolcation data for the website visitor from the browser
			visitorLatitude = position.coords.latitude;
			visitorLongitude = position.coords.longitude;	
				
			positionData = position;	
			
		    // Init the Google map
			initialize();							
		} 
	}
		
	/**
	  Error codes for HTML5 geolocation data
	 */
	function onError(err) {

		// error.code can be:
		//   0: unknown error
		//   1: permission denied
		//   2: position unavailable (error response from locaton provider)
		//   3: timed out
		switch (err.code) {
			case 0:				
				errorCode = 0;
				break;
			case 1:
				alert("The user said no!");
				errorCode = 1;
				break;
			case 2:			
				errorCode = 2;
				break;
			case 3:				
				errorCode = 3;
				break;
		}
		
		// If htm5 API fails try the fallback
		if ((errorCode != null) && (errorCode != 1)) {
		
			visitorLatitude = geoip_latitude();
			visitorLongitude = geoip_longitude();	
			positionData = null;
			initialize();			
		}
	}		
});

function getGeoData(position) {
		
	if (Modernizr.geolocation && position != null) {
			
		// These come back as null if not available
		latitude = (position.coords.latitude == null) ? "Not Available" : position.coords.latitude; 
		longitude = (position.coords.longitude == null) ? "Not Available" : position.coords.longitude; 
		altitude = (position.coords.altitude == null) ? "Not Available" : position.coords.altitude; 
		accuracy = (position.coords.accuracy == null) ? "Not Available" : position.coords.accuracy;
		altitudeAccuracy = (position.coords.altitudeAccuracy == null) ? "Not Available" : position.coords.altitudeAccuracy;
		speed = (position.coords.speed == null) ? "Not Available" : position.coords.speed;
		timestamp = (position.timestamp == null) ? "Not Available" : position.timestamp;
					
	} else {
							
		latitude = (geoip_latitude() == null) ? "Not Available" : geoip_latitude(); 
		longitude = (geoip_longitude() == null) ? "Not Available" : geoip_longitude(); 
		altitude = "Not Available"; 
		accuracy = "Not Available";
		altitudeAccuracy = "Not Available";
		speed = "Not Available";
		timestamp = "Not Available";
	}	
}

function initialize() {	
	
	markers = new Array();
	  		
	var mapOptions = {
	  zoom: 12,
	  panControl: true,
	  zoomControl: true,
	  mapTypeControl: true,
	  scaleControl: true,
	  streetViewControl: true,
	  overviewMapControl: true,
	  center: new google.maps.LatLng(visitorLatitude, visitorLongitude),
	  mapTypeId: google.maps.MapTypeId.ROADMAP
	};
	
	map = new google.maps.Map(document.getElementById('map'), mapOptions);
	markerBounds = new google.maps.LatLngBounds();
							
	// Display some geolocation data to the web page
	getGeoData(positionData);	
	
	// Content for map speech bubble
	var htmlYouAreHere = createVisitorContent();
		
	visitorLatlng = new google.maps.LatLng(visitorLatitude, visitorLongitude);	
	currentLocation = visitorLatlng;
	
	visitorInfowindow = new google.maps.InfoWindow({
		content: htmlYouAreHere,		 
	});
		
	var visitorIcon = "images/home_icon.png";

	var visitorMarker = new google.maps.Marker({
		position: visitorLatlng,
		map: map,		
		animation: google.maps.Animation.DROP,
		icon: visitorIcon
	});
		
	// Create click handler for home icon
	google.maps.event.addListener(visitorMarker, 'click', function() {
	
		visitorInfowindow.open(map, visitorMarker);
		currentLocation = visitorLatlng;
		clearPlaces();
		
		// Clear any user generated 
		// markers that may be visible
		if (newMarker) {
			newMarker.setMap(null);
		}
		
		newMarker = null;
	});	
	
	newLocationIcon = "images/new_location_icon.png";
	
	function placeMarker(location) {
	
		markerBounds = new google.maps.LatLngBounds();
	
		visitorInfowindow.close();	
		currentLocation = location;
		enabled = true;
	
		if (newMarker) {
		
			newMarker.setPosition(location);
			
		} else {
		
			newMarker = new google.maps.Marker({
				position: location,
				map: map,
				icon: newLocationIcon
			});		
		}

		// Create click handler for new user generated marker
		google.maps.event.addListener(newMarker, 'click', function() {	
		
			if (enabled) {
				getPlaces();
			}
			
			enabled = false;
		});	

		// Store data to marker object that will go into local storage
		if (Modernizr.localstorage) { 
		
			var targetData = {};	
			targetData.latitude = location.lat().toString();
			targetData.longitude = location.lng().toString();
			setLocalStorageItem("target", JSON.stringify(targetData));	
		}
	}

	google.maps.event.addListener(map, 'click', function(event) {

		if (!zoomingIn) {
		
			clearPlaces();
			placeMarker(event.latLng);			
		}
	}); 
		
    // Create the DIV to hold the control and call the HomeControl() constructor
    // passing in this DIV.
    var homeControlDiv = document.createElement('div');
    var homeControl = new HomeControl(homeControlDiv, map);

    homeControlDiv.index = 1;
    map.controls[google.maps.ControlPosition.TOP_RIGHT].push(homeControlDiv);
	
	if (Modernizr.localstorage) { 
	
		var length = localStorage.length;
      
		for (i = 0 ; i < length ; i++) {
		
            var key = localStorage.key(i);
			
            if (key.indexOf("marker") === 0) {
              
                var rawData = getLocalStorageItem(key);
				
                // Data parsed here as it was stringified before saving
                createStoredPlaceMarker(JSON.parse(rawData));
				
            } else if (key.indexOf("target") === 0) {
			
				var rawData = getLocalStorageItem(key);
			
				createStoredTargetMarker(JSON.parse(rawData));				
			}
        }
	}
}

function createStoredTargetMarker(data) {

	// Get the place location
	var placeLoc = new google.maps.LatLng(data.latitude, data.longitude);
	var newLocationIcon = "images/new_location_icon.png";

	newMarker = new google.maps.Marker({
		position: placeLoc,
		map: map,
		icon: newLocationIcon
	});
}

function createStoredPlaceMarker(data) {

	// Create the info window
    infowindow = new google.maps.InfoWindow();
	
	// Get the place location
	var placeLoc = new google.maps.LatLng(data.latitude, data.longitude);
	var fishingLocationIcon = "images/fish.png";
		
	// Create marker object
	var marker = new google.maps.Marker({
	    map: map,
	    position: placeLoc,
		animation: google.maps.Animation.DROP,
		icon: fishingLocationIcon
	});
	
	// Extend markerBounds with each point added to the map.
	markerBounds.extend(placeLoc);
	
	// Finally we can call the Map.fitBounds() method to set the map to fit
	// our markerBounds
	map.fitBounds(markerBounds);
	
	// Store reference to the markers
	// so that we can find out which one is clicked
	markers.push(marker);
	
	// Click handler to open and populate the info window for this place
	google.maps.event.addListener(marker, 'click', function() {
		zoomingIn = false;
		infowindow.setContent(data.windowContent);
	    infowindow.open(map, this);
	});	
}

function HomeControl(controlDiv, map) {

    // Set CSS styles for the DIV containing the control
    // Setting padding to 5 px will offset the control
    // from the edge of the map.
    controlDiv.style.padding = '5px';

    // Set CSS for the control border.
    var controlUI = document.createElement('div');
    controlUI.style.backgroundColor = '#ffffff';
    controlUI.style.borderStyle = 'solid';
    controlUI.style.borderColor = '#000000';  
    controlUI.style.borderWidth = '1px';
    controlUI.style.cursor = 'pointer';
    controlUI.style.textAlign = 'center';
    controlDiv.appendChild(controlUI);

    // Set CSS for the control interior.
    var controlText = document.createElement('div');
    controlText.style.fontFamily = 'Arial,sans-serif';
    controlText.style.fontSize = '13px';
    controlText.style.color = '#000000';
    controlText.style.paddingBottom = '1px';
    controlText.style.paddingTop = '1px';
    controlText.style.paddingLeft = '8px';
	controlText.style.paddingRight = '8px';
    controlText.innerHTML = '<strong>Help<strong>';
    controlUI.appendChild(controlText);

    var legend = '<h2>Introduction</h2>'
           + '<p>My Fishing Places helps you find coarse fishing locations within a 30 mile radius of a given location. When you first open the application it auto-detects your geolocation and adds a "home" marker on the map. Clicking on this brings up your geolocation data.</p>'
           + '<h2>Setting A New Location</h2>'
           + '<p>You can set a new location simply by clicking anywhere on the map to set a new "target" marker. You can also set a new location by filling in a valid postcode and clicking the "Set New Location" button.</p>'
           + '<h2>Finding Places</h2>'
           + '<p>Clicking on the "Find Places" button will add new "fish" markers representing places that the application has found for you. These are also clickable, showing an information bubble with further information links, a route planner, and a zoom facility. You can also find places for a new location by clicking on the map once to set the location, then clicking on the new "target" marker once to find places. All place markers on the map when exiting the application will be remembered for your next visit.</p>'
         
          
	var controlLegend = document.createElement('DIV');
	$(controlLegend).addClass('gmap-control-legend');
	$(controlLegend).html(legend);
	$(controlLegend).hide();
	$(controlDiv).append(controlLegend);
			  
	// Set hover toggle event
	$(controlUI)
		.mouseenter(function() {
			$(controlLegend).slideToggle('fast');	
		})
		.mouseleave(function() {
			$(controlLegend).slideToggle('fast');
		});
}

function resetMap() {

	if (Modernizr.localstorage) { 
		localStorage.clear();
	}	

	initialize(); 
	newMarker = null;
	showLocations = false;
	zoomingIn = false;
	markerBounds = new google.maps.LatLngBounds();
	$('#address').val("");	
    enabled = true;	
}

function zoomMapIn() {
	zoomingIn = true;
	map.setZoom(18);
	map.setCenter(destinationLocation);
	map.setMapTypeId(google.maps.MapTypeId.SATELLITE);
    infowindow.close();	
}

function getRoutePlan() {

	var destinationPoint = new LatLon(destinationLocation.lat(), destinationLocation.lng());   
	var destinationGridref = OsGridRef.latLongToOsGrid(destinationPoint);	
	var currentPoint = new LatLon(currentLocation.lat(), currentLocation.lng());   
	var currentLocationGridref = OsGridRef.latLongToOsGrid(currentPoint);    

	//Link to Transport Direct website to get route with supplied co-ordinates
	var url = "http://www.transportdirect.info/transportdirect/en/journeyplanning/jplandingpage.aspx?id=BusinessLinks&do=en&dn=Your Destination&d=" + destinationGridref.easting + "," + destinationGridref.northing + "&oo=en&on=Your Location&o=" + currentLocationGridref.easting + "," + currentLocationGridref.northing + "&p=1&c=1";
	var windowName = "Route_Plan";
	var windowSize = "width=900,height=700";

	//Open the window
	window.open(url, windowName, windowSize);
}
 
